"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const PredictableSupportCode_1 = __importDefault(require("../src/PredictableSupportCode"));
const messages_1 = require("@cucumber/messages");
const fake_cucumber_1 = require("@cucumber/fake-cucumber");
describe('PredictableSupportCode', () => {
    context('#addPredictableBeforeHook', () => {
        const supportCode = new fake_cucumber_1.SupportCode();
        const predictableSupportCode = new PredictableSupportCode_1.default(supportCode);
        const scenarioId = 'some-scenario-id';
        const pickle = messages_1.messages.Pickle.create({
            astNodeIds: [scenarioId],
        });
        beforeEach(() => {
            predictableSupportCode.addPredictableBeforeHook('some/where:7', scenarioId, 'passed');
        });
        it('adds a beforeHook', () => {
            assert_1.default.equal(supportCode.beforeHooks.length, 1);
        });
        it('adds a beforeHook that matches the pickle from the scenario', () => {
            const hook = supportCode.beforeHooks[0];
            assert_1.default.notEqual(hook.match(pickle), null);
        });
        it('adds a beforeHook with correct location', () => {
            const hook = supportCode.beforeHooks[0];
            assert_1.default.deepEqual(hook.toMessage().hook.sourceReference, messages_1.messages.SourceReference.create({
                uri: 'some/where',
                location: messages_1.messages.Location.create({
                    line: 7,
                }),
            }));
        });
        it('adds a hook with a passed status when status is "passed"', () => {
            const hook = supportCode.beforeHooks[0];
            assert_1.default.equal(hook.status, messages_1.messages.TestStepFinished.TestStepResult.Status.PASSED);
        });
        it('adds a hook with a failed status when status is "failed"', () => {
            const supportCode = new fake_cucumber_1.SupportCode();
            const predictableSupportCode2 = new PredictableSupportCode_1.default(supportCode);
            predictableSupportCode2.addPredictableBeforeHook('some/where:7', scenarioId, 'failed', 0.01, 'BOOM !!');
            const hook = supportCode.beforeHooks[0];
            assert_1.default.equal(hook.status, messages_1.messages.TestStepFinished.TestStepResult.Status.FAILED);
            assert_1.default.equal(hook.errorMessage, 'BOOM !!');
            assert_1.default.equal(hook.duration, 0.01);
        });
    });
    context('#addPredictableAfterHook', () => {
        const supportCode = new fake_cucumber_1.SupportCode();
        const predictableSupportCode2 = new PredictableSupportCode_1.default(supportCode);
        const scenarioId = 'some-scenario-id';
        const pickle = messages_1.messages.Pickle.create({
            astNodeIds: [scenarioId],
        });
        beforeEach(() => {
            predictableSupportCode2.addPredictableAfterHook('some/where:7', scenarioId, 'passed');
        });
        it('adds a beforeHook', () => {
            assert_1.default.equal(supportCode.afterHooks.length, 1);
        });
        it('adds a beforeHook that matches the pickle from the scenario', () => {
            const hook = supportCode.afterHooks[0];
            assert_1.default.notEqual(hook.match(pickle), null);
        });
        it('adds a beforeHook with correct location', () => {
            const hook = supportCode.afterHooks[0];
            assert_1.default.deepEqual(hook.toMessage().hook.sourceReference, messages_1.messages.SourceReference.create({
                uri: 'some/where',
                location: messages_1.messages.Location.create({
                    line: 7,
                }),
            }));
        });
        it('adds a hook with a passed status when status is "passed"', () => {
            const hook = supportCode.afterHooks[0];
            assert_1.default.equal(hook.status, messages_1.messages.TestStepFinished.TestStepResult.Status.PASSED);
        });
        it('adds a hook with a failed status when status is "failed"', () => {
            const supportCode = new fake_cucumber_1.SupportCode();
            const predictableSupportCode2 = new PredictableSupportCode_1.default(supportCode);
            predictableSupportCode2.addPredictableAfterHook('some/where:7', scenarioId, 'failed', 0.12, 'BOOM !!');
            const hook = supportCode.afterHooks[0];
            assert_1.default.equal(hook.status, messages_1.messages.TestStepFinished.TestStepResult.Status.FAILED);
            assert_1.default.equal(hook.errorMessage, 'BOOM !!');
            assert_1.default.equal(hook.duration, 0.12);
        });
    });
    context('.addPredictableStepDefinition', () => {
        const supportCode = new fake_cucumber_1.SupportCode();
        const predictableSupportCode = new PredictableSupportCode_1.default(supportCode);
        const stepId = 'some-step-id';
        const picklestep = messages_1.messages.Pickle.PickleStep.create({
            astNodeIds: [stepId],
        });
        beforeEach(() => {
            predictableSupportCode.addPredictableStepDefinition('somewhere/over/the/rain.bow:2', stepId, 'passed');
        });
        it('registers the step definition', () => {
            assert_1.default.equal(supportCode.stepDefinitions.length, 1);
        });
        it('creates a stepDefinition that matches the step id', () => {
            const stepDefinition = supportCode.stepDefinitions[0];
            assert_1.default.notEqual(stepDefinition.match(picklestep), null);
        });
        it('creates a stepDefinition with correct location', () => {
            const stepDefinition = supportCode.stepDefinitions[0];
            assert_1.default.deepEqual(stepDefinition.toMessage().stepDefinition.sourceReference, messages_1.messages.SourceReference.create({
                uri: 'somewhere/over/the/rain.bow',
                location: messages_1.messages.Location.create({
                    line: 2,
                }),
            }));
        });
        context('when a status is provided', () => {
            it('creates a StepDefinition with status passed for "passed"', () => {
                const supportCode = new fake_cucumber_1.SupportCode();
                const predictableSupportCode = new PredictableSupportCode_1.default(supportCode);
                predictableSupportCode.addPredictableStepDefinition('somewhere/over/the/rain.bow:2', stepId, 'passed');
                const stepDefinition = supportCode.stepDefinitions[0];
                assert_1.default.equal(stepDefinition.status, messages_1.messages.TestStepFinished.TestStepResult.Status.PASSED);
            });
            it('creates a StepDefinition with status pending for "pending"', () => {
                const supportCode = new fake_cucumber_1.SupportCode();
                const predictableSupportCode = new PredictableSupportCode_1.default(supportCode);
                predictableSupportCode.addPredictableStepDefinition('somewhere/over/the/rain.bow:2', stepId, 'pending');
                const stepDefinition = supportCode.stepDefinitions[0];
                assert_1.default.equal(stepDefinition.status, messages_1.messages.TestStepFinished.TestStepResult.Status.PENDING);
            });
            it('creates a StepDefinition with status failed for "failed"', () => {
                const supportCode = new fake_cucumber_1.SupportCode();
                const predictableSupportCode = new PredictableSupportCode_1.default(supportCode);
                predictableSupportCode.addPredictableStepDefinition('somewhere/over/the/rain.bow:2', stepId, 'failed', 1.23, 'something wrong happened');
                const stepDefinition = supportCode.stepDefinitions[0];
                assert_1.default.equal(stepDefinition.status, messages_1.messages.TestStepFinished.TestStepResult.Status.FAILED);
                assert_1.default.equal(stepDefinition.duration, 1.23);
            });
        });
    });
});
//# sourceMappingURL=PredictableSupportCodeTest.js.map