"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const ts_sinon_1 = require("ts-sinon");
const JSONTraverse_1 = require("../../src/cucumber-js/JSONTraverse");
const messages_1 = require("@cucumber/messages");
describe('traversing elements', () => {
    context('traverseFeature', () => {
        it('create ');
    });
    context('traverseElement', () => {
        const simpleScenario = {
            id: 'my-scenario',
            line: 3,
            keyword: 'scenario',
            type: 'scenario',
            name: 'My scenario',
            description: 'This scenario does things',
            steps: [
                {
                    keyword: 'Given ',
                    line: 6,
                    name: 'a failed step',
                    result: {
                        status: 'failed',
                    },
                    match: {
                        location: 'whatever.go:123',
                    },
                },
            ],
        };
        const hookedScenario = {
            id: 'my-scenario',
            line: 3,
            keyword: 'scenario',
            type: 'scenario',
            name: 'My scenario',
            description: 'This scenario does things',
            steps: [
                {
                    hidden: true,
                    keyword: 'Before ',
                    line: 3,
                    name: '',
                    result: {
                        status: 'passed',
                        duration: 4560000,
                    },
                    match: {
                        location: 'whatever.go:123',
                    },
                },
                {
                    keyword: 'Given ',
                    line: 6,
                    name: 'a failed step',
                    result: {
                        status: 'passed',
                        duration: 456000,
                    },
                    match: {
                        location: 'whatever.go:123',
                    },
                },
                {
                    hidden: true,
                    keyword: 'After',
                    line: 3,
                    name: '',
                    result: {
                        status: 'failed',
                        error_message: 'This hook failed',
                        duration: 45600000,
                    },
                    match: {
                        location: 'whatever.go:456',
                    },
                },
            ],
        };
        it('creates a Scenario using the AstMaker', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            const step = messages_1.messages.GherkinDocument.Feature.Step.create();
            astMaker.makeStep.returns(step);
            JSONTraverse_1.traverseElement(simpleScenario, astMaker, () => 'new-id', supportCode);
            assert_1.default.deepEqual(astMaker.makeScenarioFeatureChild.getCall(0).args, [
                'new-id',
                3,
                'scenario',
                'My scenario',
                'This scenario does things',
                [step],
                undefined,
            ]);
        });
        it('doe not create steps for hooks', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            astMaker.makeScenarioFeatureChild.returns(messages_1.messages.GherkinDocument.Feature.FeatureChild.create({
                scenario: messages_1.messages.GherkinDocument.Feature.Scenario.create({
                    id: 'scenario-id',
                }),
            }));
            JSONTraverse_1.traverseElement(hookedScenario, astMaker, () => 'new-id', supportCode);
            assert_1.default.equal(astMaker.makeStep.callCount, 1);
        });
        it('correctly registers beforeHooks', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            astMaker.makeScenarioFeatureChild.returns(messages_1.messages.GherkinDocument.Feature.FeatureChild.create({
                scenario: messages_1.messages.GherkinDocument.Feature.Scenario.create({
                    id: 'scenario-id',
                }),
            }));
            JSONTraverse_1.traverseElement(hookedScenario, astMaker, () => 'new-id', supportCode);
            assert_1.default.deepEqual(supportCode.addPredictableBeforeHook.getCall(0).args, [
                'whatever.go:123',
                'scenario-id',
                'passed',
                4.56,
                undefined,
            ]);
        });
        it('correctly registers afterHooks', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            astMaker.makeScenarioFeatureChild.returns(messages_1.messages.GherkinDocument.Feature.FeatureChild.create({
                scenario: messages_1.messages.GherkinDocument.Feature.Scenario.create({
                    id: 'scenario-id',
                }),
            }));
            JSONTraverse_1.traverseElement(hookedScenario, astMaker, () => 'new-id', supportCode);
            assert_1.default.deepEqual(supportCode.addPredictableAfterHook.getCall(0).args, [
                'whatever.go:456',
                'scenario-id',
                'failed',
                45.6,
                'This hook failed',
            ]);
        });
    });
    context('traverseBeforeHook', () => {
        const beforeHook = {
            hidden: true,
            keyword: 'Before ',
            line: 15,
            name: '',
            result: {
                status: 'passed',
                duration: 4560000,
            },
            match: {
                location: 'whatever.go:123',
            },
        };
        it('registers the before Hook with the correct data', () => {
            const supportCode = ts_sinon_1.stubInterface();
            JSONTraverse_1.traverseBeforeHook(beforeHook, messages_1.messages.GherkinDocument.Feature.Scenario.create({ id: 'scenario-id' }), supportCode);
            assert_1.default.deepEqual(supportCode.addPredictableBeforeHook.getCall(0).args, [
                'whatever.go:123',
                'scenario-id',
                'passed',
                4.56,
                undefined,
            ]);
        });
    });
    context('traverseAfterHook', () => {
        const beforeHook = {
            hidden: true,
            keyword: 'After ',
            line: 15,
            name: '',
            result: {
                status: 'passed',
                duration: 1230000,
            },
            match: {
                location: 'whatever.go:123',
            },
        };
        it('registers the after Hook with the correct data', () => {
            const supportCode = ts_sinon_1.stubInterface();
            JSONTraverse_1.traverseAfterHook(beforeHook, messages_1.messages.GherkinDocument.Feature.Scenario.create({ id: 'scenario-id' }), supportCode);
            assert_1.default.deepEqual(supportCode.addPredictableAfterHook.getCall(0).args, [
                'whatever.go:123',
                'scenario-id',
                'passed',
                1.23,
                undefined,
            ]);
        });
    });
    context('traverseStep', () => {
        const simpleStep = {
            keyword: 'Given ',
            line: 15,
            name: 'a step with a doctring:',
            result: {
                status: 'failed',
                duration: 456000,
            },
            match: {
                location: 'whatever.go:123',
            },
        };
        const docStringStep = {
            arguments: [
                {
                    content: '  This is some content',
                    line: 16,
                },
            ],
            keyword: 'Given ',
            line: 15,
            name: 'a step with a doctring:',
            result: {
                status: 'undefined',
            },
        };
        const datatableStep = {
            arguments: [
                {
                    rows: [
                        {
                            cells: ['name', 'value'],
                        },
                        {
                            cells: ['plic', '0'],
                        },
                        {
                            cells: ['ploc', '1'],
                        },
                    ],
                },
            ],
            keyword: 'Given ',
            line: 15,
            name: 'a step with a doctring:',
            result: {
                status: 'undefined',
            },
        };
        it('builds the doctring using the AstMaker', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            const docString = messages_1.messages.GherkinDocument.Feature.Step.DocString.create();
            astMaker.makeDocstring.returns(docString);
            JSONTraverse_1.traverseStep(docStringStep, astMaker, () => 'the-id', supportCode);
            assert_1.default.deepEqual(astMaker.makeStep.getCall(0).args, [
                'the-id',
                15,
                'Given ',
                'a step with a doctring:',
                docString,
                null,
            ]);
        });
        it('builds the datatable using the astMaker', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            const datatable = messages_1.messages.GherkinDocument.Feature.Step.DataTable.create();
            astMaker.makeDataTable.returns(datatable);
            JSONTraverse_1.traverseStep(datatableStep, astMaker, () => 'the-id', supportCode);
            assert_1.default.deepEqual(astMaker.makeStep.getCall(0).args, [
                'the-id',
                15,
                'Given ',
                'a step with a doctring:',
                null,
                datatable,
            ]);
        });
        it('registers the step in supportCode', () => {
            const astMaker = ts_sinon_1.stubInterface();
            const supportCode = ts_sinon_1.stubInterface();
            astMaker.makeStep.returns(messages_1.messages.GherkinDocument.Feature.Step.create({
                id: 'a-random-step-id',
            }));
            JSONTraverse_1.traverseStep(simpleStep, astMaker, () => 'the-id', supportCode);
            assert_1.default.deepEqual(supportCode.addPredictableStepDefinition.getCall(0).args, [
                'whatever.go:123',
                'a-random-step-id',
                'failed',
                0.456,
                undefined,
            ]);
        });
    });
    context('traverseDocString', () => {
        const docString = {
            content: '  This is some content',
            line: 16,
        };
        it('call AstMaker.makeDocString with the content but no media-type', () => {
            const astMaker = ts_sinon_1.stubInterface();
            JSONTraverse_1.traverseDocString(docString, astMaker);
            assert_1.default.deepEqual(astMaker.makeDocstring.getCall(0).args, [null, '  This is some content']);
        });
    });
});
//# sourceMappingURL=JSONTraverseTest.js.map